@@ -136,17 +136,17 @@ static void dump_values(grib_dumper* d, grib_accessor* a)
     int err                               = 0;
     int r                                 = 0;
     long count                            = 0;
-    grib_context* c                       = a->context;
+    grib_context* c                       = a->context_;
     grib_handle* h                        = grib_handle_of_accessor(a);
 
-    if ((a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0 || (a->flags & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0)
+    if ((a->flags_ & GRIB_ACCESSOR_FLAG_DUMP) == 0 || (a->flags_ & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0)
         return;
 
-    grib_value_count(a, &count);
+    a->value_count(&count);
     size = count;
 
     if (size <= 1) {
-        err = grib_unpack_double(a, &value, &size);
+        err = a->unpack_double(&value, &size);
     }
 
     self->empty = 0;
@@ -154,18 +154,18 @@ static void dump_values(grib_dumper* d, grib_accessor* a)
     if (size > 1) {
         depth -= 2;
 
-        if ((r = compute_bufr_key_rank(h, self->keys, a->name)) != 0)
-            fprintf(self->dumper.out, "  call codes_get(ibufr, '#%d#%s', rValues)\n", r, a->name);
+        if ((r = compute_bufr_key_rank(h, self->keys, a->name_)) != 0)
+            fprintf(self->dumper.out, "  call codes_get(ibufr, '#%d#%s', rValues)\n", r, a->name_);
         else
-            fprintf(self->dumper.out, "  call codes_get(ibufr, '%s', rValues)\n", a->name);
+            fprintf(self->dumper.out, "  call codes_get(ibufr, '%s', rValues)\n", a->name_);
     }
     else {
-        r = compute_bufr_key_rank(h, self->keys, a->name);
+        r = compute_bufr_key_rank(h, self->keys, a->name_);
         if (!grib_is_missing_double(a, value)) {
             if (r != 0)
-                fprintf(self->dumper.out, "  call codes_get(ibufr, '#%d#%s', rVal)\n", r, a->name);
+                fprintf(self->dumper.out, "  call codes_get(ibufr, '#%d#%s', rVal)\n", r, a->name_);
             else
-                fprintf(self->dumper.out, "  call codes_get(ibufr, '%s', rVal)\n", a->name);
+                fprintf(self->dumper.out, "  call codes_get(ibufr, '%s', rVal)\n", a->name_);
         }
     }
 
@@ -174,12 +174,12 @@ static void dump_values(grib_dumper* d, grib_accessor* a)
         int dofree = 0;
 
         if (r != 0) {
-            prefix = (char*)grib_context_malloc_clear(c, sizeof(char) * (strlen(a->name) + 10));
+            prefix = (char*)grib_context_malloc_clear(c, sizeof(char) * (strlen(a->name_) + 10));
             dofree = 1;
-            snprintf(prefix, 1024, "#%d#%s", r, a->name);
+            snprintf(prefix, 1024, "#%d#%s", r, a->name_);
         }
         else
-            prefix = (char*)a->name;
+            prefix = (char*)a->name_;
 
         dump_attributes(d, a, prefix);
         if (dofree)
@@ -197,34 +197,34 @@ static void dump_values_attribute(grib_dumper* d, grib_accessor* a, const char*
     size_t size                           = 0;
     int err                               = 0;
     long count                            = 0;
-    grib_context* c                       = a->context;
+    grib_context* c                       = a->context_;
 
-    if ((a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0 || (a->flags & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0)
+    if ((a->flags_ & GRIB_ACCESSOR_FLAG_DUMP) == 0 || (a->flags_ & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0)
         return;
 
-    grib_value_count(a, &count);
+    a->value_count(&count);
     size = count;
 
     if (size <= 1) {
-        err = grib_unpack_double(a, &value, &size);
+        err = a->unpack_double(&value, &size);
     }
 
     self->empty = 0;
 
     if (size > 1) {
-        fprintf(self->dumper.out, "  call codes_get(ibufr, '%s->%s', rValues)\n", prefix, a->name);
+        fprintf(self->dumper.out, "  call codes_get(ibufr, '%s->%s', rValues)\n", prefix, a->name_);
     }
     else {
         if (!grib_is_missing_double(a, value)) {
-            fprintf(self->dumper.out, "  call codes_get(ibufr, '%s->%s', rVal)\n", prefix, a->name);
+            fprintf(self->dumper.out, "  call codes_get(ibufr, '%s->%s', rVal)\n", prefix, a->name_);
         }
     }
 
     if (self->isLeaf == 0) {
         char* prefix1;
 
-        prefix1 = (char*)grib_context_malloc_clear(c, sizeof(char) * (strlen(a->name) + strlen(prefix) + 5));
-        snprintf(prefix1, 1024, "%s->%s", prefix, a->name);
+        prefix1 = (char*)grib_context_malloc_clear(c, sizeof(char) * (strlen(a->name_) + strlen(prefix) + 5));
+        snprintf(prefix1, 1024, "%s->%s", prefix, a->name_);
 
         dump_attributes(d, a, prefix1);
 
@@ -243,28 +243,28 @@ static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment)
     int err                               = 0;
     int r                                 = 0;
     long count                            = 0;
-    grib_context* c                       = a->context;
+    grib_context* c                       = a->context_;
     grib_handle* h                        = grib_handle_of_accessor(a);
 
-    if ((a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0)
+    if ((a->flags_ & GRIB_ACCESSOR_FLAG_DUMP) == 0)
         return;
 
-    grib_value_count(a, &count);
+    a->value_count(&count);
     size = count;
 
-    if ((a->flags & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0) {
+    if ((a->flags_ & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0) {
         if (self->isLeaf == 0) {
             char* prefix;
             int dofree = 0;
 
-            r = compute_bufr_key_rank(h, self->keys, a->name);
+            r = compute_bufr_key_rank(h, self->keys, a->name_);
             if (r != 0) {
-                prefix = (char*)grib_context_malloc_clear(c, sizeof(char) * (strlen(a->name) + 10));
+                prefix = (char*)grib_context_malloc_clear(c, sizeof(char) * (strlen(a->name_) + 10));
                 dofree = 1;
-                snprintf(prefix, 1024, "#%d#%s", r, a->name);
+                snprintf(prefix, 1024, "#%d#%s", r, a->name_);
             }
             else
-                prefix = (char*)a->name;
+                prefix = (char*)a->name_;
 
             dump_attributes(d, a, prefix);
             if (dofree)
@@ -275,7 +275,7 @@ static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment)
     }
 
     if (size <= 1) {
-        err = grib_unpack_long(a, &value, &size);
+        err = a->unpack_long(&value, &size);
     }
 
     self->empty = 0;
@@ -284,18 +284,18 @@ static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment)
         depth -= 2;
         fprintf(self->dumper.out, "  if(allocated(iValues)) deallocate(iValues)\n");
 
-        if ((r = compute_bufr_key_rank(h, self->keys, a->name)) != 0)
-            fprintf(self->dumper.out, "  call codes_get(ibufr, '#%d#%s', iValues)\n", r, a->name);
+        if ((r = compute_bufr_key_rank(h, self->keys, a->name_)) != 0)
+            fprintf(self->dumper.out, "  call codes_get(ibufr, '#%d#%s', iValues)\n", r, a->name_);
         else
-            fprintf(self->dumper.out, "  call codes_get(ibufr, '%s', iValues)\n", a->name);
+            fprintf(self->dumper.out, "  call codes_get(ibufr, '%s', iValues)\n", a->name_);
     }
     else {
-        r = compute_bufr_key_rank(h, self->keys, a->name);
+        r = compute_bufr_key_rank(h, self->keys, a->name_);
         if (!grib_is_missing_long(a, value)) {
             if (r != 0)
-                fprintf(self->dumper.out, "  call codes_get(ibufr, '#%d#%s', iVal)\n", r, a->name);
+                fprintf(self->dumper.out, "  call codes_get(ibufr, '#%d#%s', iVal)\n", r, a->name_);
             else
-                fprintf(self->dumper.out, "  call codes_get(ibufr, '%s', iVal)\n", a->name);
+                fprintf(self->dumper.out, "  call codes_get(ibufr, '%s', iVal)\n", a->name_);
         }
     }
 
@@ -304,12 +304,12 @@ static void dump_long(grib_dumper* d, grib_accessor* a, const char* comment)
         int dofree = 0;
 
         if (r != 0) {
-            prefix = (char*)grib_context_malloc_clear(c, sizeof(char) * (strlen(a->name) + 10));
+            prefix = (char*)grib_context_malloc_clear(c, sizeof(char) * (strlen(a->name_) + 10));
             dofree = 1;
-            snprintf(prefix,  1024,"#%d#%s", r, a->name);
+            snprintf(prefix,  1024,"#%d#%s", r, a->name_);
         }
         else
-            prefix = (char*)a->name;
+            prefix = (char*)a->name_;
 
         dump_attributes(d, a, prefix);
         if (dofree)
@@ -326,16 +326,16 @@ static void dump_long_attribute(grib_dumper* d, grib_accessor* a, const char* pr
     size_t size                           = 0;
     int err                               = 0;
     long count                            = 0;
-    grib_context* c                       = a->context;
+    grib_context* c                       = a->context_;
 
-    if ((a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0 || (a->flags & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0)
+    if ((a->flags_ & GRIB_ACCESSOR_FLAG_DUMP) == 0 || (a->flags_ & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0)
         return;
 
-    grib_value_count(a, &count);
+    a->value_count(&count);
     size = count;
 
     if (size <= 1) {
-        err = grib_unpack_long(a, &value, &size);
+        err = a->unpack_long(&value, &size);
     }
 
     self->empty = 0;
@@ -344,19 +344,21 @@ static void dump_long_attribute(grib_dumper* d, grib_accessor* a, const char* pr
         depth -= 2;
         fprintf(self->dumper.out, "  if(allocated(iValues)) deallocate(iValues)\n");
 
-        fprintf(self->dumper.out, "  call codes_get(ibufr, '%s->%s', iValues)\n", prefix, a->name);
+        fprintf(self->dumper.out, "  call codes_get(ibufr, '%s->%s', iValues)\n", prefix, a->name_);
     }
     else {
-        if (!grib_is_missing_long(a, value)) {
-            fprintf(self->dumper.out, "  call codes_get(ibufr, '%s->%s', iVal)\n", prefix, a->name);
+        if (!codes_bufr_key_exclude_from_dump(prefix)) {
+            if (!grib_is_missing_long(a, value)) {
+                fprintf(self->dumper.out, "  call codes_get(ibufr, '%s->%s', iVal)\n", prefix, a->name_);
+            }
         }
     }
 
     if (self->isLeaf == 0) {
         char* prefix1;
 
-        prefix1 = (char*)grib_context_malloc_clear(c, sizeof(char) * (strlen(a->name) + strlen(prefix) + 5));
-        snprintf(prefix1,  1024,"%s->%s", prefix, a->name);
+        prefix1 = (char*)grib_context_malloc_clear(c, sizeof(char) * (strlen(a->name_) + strlen(prefix) + 5));
+        snprintf(prefix1,  1024,"%s->%s", prefix, a->name_);
 
         dump_attributes(d, a, prefix1);
 
@@ -379,18 +381,18 @@ static void dump_double(grib_dumper* d, grib_accessor* a, const char* comment)
     grib_handle* h  = grib_handle_of_accessor(a);
     grib_context* c = h->context;
 
-    if ((a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0 || (a->flags & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0)
+    if ((a->flags_ & GRIB_ACCESSOR_FLAG_DUMP) == 0 || (a->flags_ & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0)
         return;
 
-    grib_unpack_double(a, &value, &size);
+    a->unpack_double(&value, &size);
     self->empty = 0;
 
-    r = compute_bufr_key_rank(h, self->keys, a->name);
+    r = compute_bufr_key_rank(h, self->keys, a->name_);
     if (!grib_is_missing_double(a, value)) {
         if (r != 0)
-            fprintf(self->dumper.out, "  call codes_get(ibufr,'#%d#%s', rVal)\n", r, a->name);
+            fprintf(self->dumper.out, "  call codes_get(ibufr,'#%d#%s', rVal)\n", r, a->name_);
         else
-            fprintf(self->dumper.out, "  call codes_get(ibufr,'%s', rVal)\n", a->name);
+            fprintf(self->dumper.out, "  call codes_get(ibufr,'%s', rVal)\n", a->name_);
     }
 
     if (self->isLeaf == 0) {
@@ -398,12 +400,12 @@ static void dump_double(grib_dumper* d, grib_accessor* a, const char* comment)
         int dofree = 0;
 
         if (r != 0) {
-            prefix = (char*)grib_context_malloc_clear(c, sizeof(char) * (strlen(a->name) + 10));
+            prefix = (char*)grib_context_malloc_clear(c, sizeof(char) * (strlen(a->name_) + 10));
             dofree = 1;
-            snprintf(prefix, 1024, "#%d#%s", r, a->name);
+            snprintf(prefix, 1024, "#%d#%s", r, a->name_);
         }
         else
-            prefix = (char*)a->name;
+            prefix = (char*)a->name_;
 
         dump_attributes(d, a, prefix);
         if (dofree)
@@ -422,12 +424,12 @@ static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comm
     int r                                 = 0;
     grib_handle* h                        = grib_handle_of_accessor(a);
 
-    c = a->context;
+    c = a->context_;
 
-    if ((a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0 || (a->flags & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0)
+    if ((a->flags_ & GRIB_ACCESSOR_FLAG_DUMP) == 0 || (a->flags_ & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0)
         return;
 
-    grib_value_count(a, &count);
+    a->value_count(&count);
     size = count;
     if (size == 1) {
         dump_string(d, a, comment);
@@ -440,10 +442,10 @@ static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comm
     self->empty = 0;
 
     if (self->isLeaf == 0) {
-        if ((r = compute_bufr_key_rank(h, self->keys, a->name)) != 0)
-            fprintf(self->dumper.out, "  call codes_get_string_array(ibufr,'#%d#%s',sValues)\n", r, a->name);
+        if ((r = compute_bufr_key_rank(h, self->keys, a->name_)) != 0)
+            fprintf(self->dumper.out, "  call codes_get_string_array(ibufr,'#%d#%s',sValues)\n", r, a->name_);
         else
-            fprintf(self->dumper.out, "  call codes_get_string_array(ibufr,'%s',sValues)\n", a->name);
+            fprintf(self->dumper.out, "  call codes_get_string_array(ibufr,'%s',sValues)\n", a->name_);
     }
 
     if (self->isLeaf == 0) {
@@ -451,12 +453,12 @@ static void dump_string_array(grib_dumper* d, grib_accessor* a, const char* comm
         int dofree = 0;
 
         if (r != 0) {
-            prefix = (char*)grib_context_malloc_clear(c, sizeof(char) * (strlen(a->name) + 10));
+            prefix = (char*)grib_context_malloc_clear(c, sizeof(char) * (strlen(a->name_) + 10));
             dofree = 1;
-            snprintf(prefix, 1024, "#%d#%s", r, a->name);
+            snprintf(prefix, 1024, "#%d#%s", r, a->name_);
         }
         else
-            prefix = (char*)a->name;
+            prefix = (char*)a->name_;
 
         dump_attributes(d, a, prefix);
         if (dofree)
@@ -474,18 +476,18 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
     char value[MAX_STRING_SIZE] = {0,}; /* See ECC-710 */
     size_t size                 = MAX_STRING_SIZE;
     char* p                     = NULL;
-    grib_context* c             = a->context;
+    grib_context* c             = a->context_;
     int r = 0, err = 0;
     grib_handle* h = grib_handle_of_accessor(a);
 
-    if ((a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0 || (a->flags & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0)
+    if ((a->flags_ & GRIB_ACCESSOR_FLAG_DUMP) == 0 || (a->flags_ & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0)
         return;
 
     self->empty = 0;
 
-    err = grib_unpack_string(a, value, &size);
+    err = a->unpack_string(value, &size);
     p   = value;
-    r   = compute_bufr_key_rank(h, self->keys, a->name);
+    r   = compute_bufr_key_rank(h, self->keys, a->name_);
     if (grib_is_missing_string(a, (unsigned char*)value, size)) {
         return;
     }
@@ -499,9 +501,9 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
     if (self->isLeaf == 0) {
         depth += 2;
         if (r != 0)
-            fprintf(self->dumper.out, "  call codes_get(ibufr, '#%d#%s', sVal)\n", r, a->name);
+            fprintf(self->dumper.out, "  call codes_get(ibufr, '#%d#%s', sVal)\n", r, a->name_);
         else
-            fprintf(self->dumper.out, "  call codes_get(ibufr, '%s', sVal)\n", a->name);
+            fprintf(self->dumper.out, "  call codes_get(ibufr, '%s', sVal)\n", a->name_);
     }
     /*fprintf(self->dumper.out,"\'%s\')\n",value);*/
 
@@ -511,12 +513,12 @@ static void dump_string(grib_dumper* d, grib_accessor* a, const char* comment)
         int dofree = 0;
 
         if (r != 0) {
-            prefix = (char*)grib_context_malloc_clear(c, sizeof(char) * (strlen(a->name) + 10));
+            prefix = (char*)grib_context_malloc_clear(c, sizeof(char) * (strlen(a->name_) + 10));
             dofree = 1;
-            snprintf(prefix, 1024, "#%d#%s", r, a->name);
+            snprintf(prefix, 1024, "#%d#%s", r, a->name_);
         }
         else
-            prefix = (char*)a->name;
+            prefix = (char*)a->name_;
 
         dump_attributes(d, a, prefix);
         if (dofree)
@@ -550,9 +552,9 @@ static void _dump_long_array(grib_handle* h, FILE* f, const char* key)
 static void dump_section(grib_dumper* d, grib_accessor* a, grib_block_of_accessors* block)
 {
     grib_dumper_bufr_decode_fortran* self = (grib_dumper_bufr_decode_fortran*)d;
-    if (strcmp(a->name, "BUFR")==0 ||
-        strcmp(a->name, "GRIB")==0 ||
-        strcmp(a->name, "META")==0) {
+    if (strcmp(a->name_, "BUFR")==0 ||
+        strcmp(a->name_, "GRIB")==0 ||
+        strcmp(a->name_, "META")==0) {
         grib_handle* h = grib_handle_of_accessor(a);
         depth          = 2;
         self->empty    = 1;
@@ -566,8 +568,8 @@ static void dump_section(grib_dumper* d, grib_accessor* a, grib_block_of_accesso
         grib_dump_accessors_block(d, block);
         depth -= 2;
     }
-    else if (strcmp(a->name, "groupNumber")==0) {
-        if ((a->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0)
+    else if (strcmp(a->name_, "groupNumber")==0) {
+        if ((a->flags_ & GRIB_ACCESSOR_FLAG_DUMP) == 0)
             return;
         self->empty = 1;
         depth += 2;
@@ -584,26 +586,26 @@ static void dump_attributes(grib_dumper* d, grib_accessor* a, const char* prefix
     int i                                 = 0;
     grib_dumper_bufr_decode_fortran* self = (grib_dumper_bufr_decode_fortran*)d;
     unsigned long flags;
-    while (i < MAX_ACCESSOR_ATTRIBUTES && a->attributes[i]) {
+    while (i < MAX_ACCESSOR_ATTRIBUTES && a->attributes_[i]) {
         self->isAttribute = 1;
-        if ((d->option_flags & GRIB_DUMP_FLAG_ALL_ATTRIBUTES) == 0 && (a->attributes[i]->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0) {
+        if ((d->option_flags & GRIB_DUMP_FLAG_ALL_ATTRIBUTES) == 0 && (a->attributes_[i]->flags_ & GRIB_ACCESSOR_FLAG_DUMP) == 0) {
             i++;
             continue;
         }
-        self->isLeaf = a->attributes[i]->attributes[0] == NULL ? 1 : 0;
-        flags        = a->attributes[i]->flags;
-        a->attributes[i]->flags |= GRIB_ACCESSOR_FLAG_DUMP;
-        switch (grib_accessor_get_native_type(a->attributes[i])) {
+        self->isLeaf = a->attributes_[i]->attributes_[0] == NULL ? 1 : 0;
+        flags        = a->attributes_[i]->flags_;
+        a->attributes_[i]->flags_ |= GRIB_ACCESSOR_FLAG_DUMP;
+        switch (a->attributes_[i]->get_native_type()) {
             case GRIB_TYPE_LONG:
-                dump_long_attribute(d, a->attributes[i], prefix);
+                dump_long_attribute(d, a->attributes_[i], prefix);
                 break;
             case GRIB_TYPE_DOUBLE:
-                dump_values_attribute(d, a->attributes[i], prefix);
+                dump_values_attribute(d, a->attributes_[i], prefix);
                 break;
             case GRIB_TYPE_STRING:
                 break;
         }
-        a->attributes[i]->flags = flags;
+        a->attributes_[i]->flags_ = flags;
         i++;
     }
     self->isLeaf      = 0;
