@@ -24,10 +24,9 @@ grib_option grib_options[] = {
     { "H", 0, "Compare only message headers. Bit-by-bit compare on. Incompatible with -c option.\n", 0, 1, 0 },
     { "R:", 0, 0, 0, 1, 0 },
     { "A:", 0, 0, 0, 1, 0 },
-    /*    {"P",0,"Compare data values using the packing error as tolerance.\n",0,1,0},*/
     { "t:", "factor", "Compare data values using factor multiplied by the tolerance specified in options -R -A.\n", 0, 1, 0 },
     { "w:", 0, 0, 0, 1, 0 },
-    { "f", 0, 0, 0, 1, 0 },
+    { "f", 0, "Forcefully compare, do not stop after first difference.\n", 0, 1, 0 },
     { "F", 0, 0, 1, 0, 0 },
     { "q", 0, 0, 1, 0, 0 },
     { "M", 0, 0, 1, 0, 0 },
@@ -50,17 +49,6 @@ const char* tool_name       = "bufr_compare";
 const char* tool_online_doc = "https://confluence.ecmwf.int/display/ECC/bufr_compare";
 const char* tool_usage      = "[options] bufr_file1 bufr_file2";
 
-GRIB_INLINE static int grib_inline_strcmp(const char* a, const char* b)
-{
-    if (*a != *b)
-        return 1;
-    while ((*a != 0 && *b != 0) && *(a) == *(b)) {
-        a++;
-        b++;
-    }
-    return (*a == 0 && *b == 0) ? 0 : 1;
-}
-
 typedef double (*compare_double_proc)(const double*, const double*, const double*);
 
 typedef struct grib_error grib_error;
@@ -74,7 +62,6 @@ struct grib_error
 static grib_error* error_summary;
 static compare_double_proc compare_double;
 static double global_tolerance     = 0;
-static int packingCompare          = 0;
 static grib_string_list* blocklist = 0;
 static grib_string_list* keys_list = NULL; /* Used to determine rank of key */
 static int isLeafKey               = 0;    /* 0 if key is top-level, 1 if key has no children attributes */
@@ -86,7 +73,6 @@ static int compare_attribute(grib_handle* handle1, grib_handle* handle2, grib_ru
 
 static int error     = 0;
 static int count     = 0;
-static int lastPrint = 0;
 static int force     = 0;
 
 /* ECC-651: Boolean 'two_way' set to 1 when '-2' option used */
@@ -238,7 +224,7 @@ int grib_tool_before_getopt(grib_runtime_options* options)
 
 int grib_tool_init(grib_runtime_options* options)
 {
-    int ret               = 0, i;
+    int ret = 0;
     grib_context* context = grib_context_get_default();
 
     options->strict = 1;
@@ -317,16 +303,6 @@ int grib_tool_init(grib_runtime_options* options)
     compare_double   = &compare_double_absolute;
     if (grib_options_on("R:")) {
         global_tolerance = 0;
-        for (i = 0; i < options->tolerance_count; i++) {
-            if (!strcmp((options->tolerance[i]).name, "all")) {
-                global_tolerance = (options->tolerance[i]).double_value;
-                break;
-            }
-            if (!strcmp((options->tolerance[i]).name, "global")) {
-                global_tolerance = (options->tolerance[i]).double_value;
-                break;
-            }
-        }
         compare_double  = &compare_double_relative;
         compareAbsolute = 0;
     }
@@ -339,18 +315,14 @@ int grib_tool_init(grib_runtime_options* options)
             global_tolerance = atof(grib_options_get_option("A:"));
         }
     }
-    if (grib_options_on("P")) {
-        packingCompare = 1;
-        compare_double = &compare_double_absolute;
-    }
 
     if (grib_options_on("t:"))
         tolerance_factor = atof(grib_options_get_option("t:"));
 
     if (grib_options_on("R:")) {
-        char* sarg               = grib_options_get_option("R:");
+        char* sarg = grib_options_get_option("R:");
         options->tolerance_count = MAX_KEYS;
-        ret                      = parse_keyval_string(tool_name, sarg, 1, GRIB_TYPE_DOUBLE, options->tolerance, &(options->tolerance_count));
+        ret = parse_keyval_string(tool_name, sarg, 1, GRIB_TYPE_DOUBLE, options->tolerance, &(options->tolerance_count));
         if (ret == GRIB_INVALID_ARGUMENT) {
             usage();
             exit(1);
@@ -365,7 +337,7 @@ int grib_tool_init(grib_runtime_options* options)
             if (path_is_directory(infile->name)) {
                 /* Take the filename of the 1st file and append to dir */
                 char bufr[2048] = {0,};
-                /* options->infile_extra->name is the 1st file */
+                /* options->infile_extra->name_ is the 1st file */
                 snprintf(bufr, sizeof(bufr), "%s%c%s",
                         infile->name,
                         get_dir_separator_char(),
@@ -395,7 +367,6 @@ int grib_tool_new_file_action(grib_runtime_options* options, grib_tools_file* fi
 static void printInfo(grib_handle* h)
 {
     printf("== %d == DIFFERENCE == ", count);
-    lastPrint = count;
 }
 
 static void print_index_key_values(grib_index* index, int icounter, const char* error_message)
@@ -531,11 +502,6 @@ int grib_tool_skip_handle(grib_runtime_options* options, grib_handle* h)
     return 0;
 }
 
-void grib_tool_print_key_values(grib_runtime_options* options, grib_handle* h)
-{
-    grib_print_key_values(options, h);
-}
-
 int grib_tool_finalise_action(grib_runtime_options* options)
 {
     grib_error* e   = error_summary;
@@ -680,7 +646,7 @@ static int strings_are_different(grib_handle* h1, grib_handle* h2, const char* k
                                  const char* s1, const char* s2,
                                  size_t slen1, size_t slen2)
 {
-    if (grib_inline_strcmp(s1, s2) == 0) {
+    if (strcmp(s1, s2) == 0) {
         return 0;
     }
     /* Strings are different. Now check if strings are 'missing'.
@@ -704,11 +670,9 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
 
     char *sval1 = NULL, *sval2 = NULL;
     char **svals1 = NULL, **svals2 = NULL;
-    unsigned char *uval1 = NULL, *uval2 = NULL;
     double *dval1 = NULL, *dval2 = NULL;
     long *lval1 = NULL, *lval2 = NULL;
     double maxdiff       = 0;
-    double packingError1 = 0, packingError2 = 0;
     double value_tolerance = 0;
     grib_context* c        = handle1->context;
     const char* first_str  = (handles_swapped == 0 ? "1st" : "2nd");
@@ -721,7 +685,7 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
 
     if (type1 == GRIB_TYPE_UNDEFINED && (err = grib_get_native_type(handle1, name, &type1)) != GRIB_SUCCESS) {
         printInfo(handle1);
-        printf("Oops... cannot get type of [%s] in %s field: %s\n", name, first_str, grib_get_error_message(err));
+        printf("Error: cannot get type of [%s] in %s field: %s\n", name, first_str, grib_get_error_message(err));
         save_error(c, name);
         return err;
     }
@@ -734,7 +698,7 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
             return err;
         }
         printInfo(handle1);
-        printf("Oops... cannot get type of [%s] in %s field: %s\n", name, second_str, grib_get_error_message(err));
+        printf("Error: cannot get type of [%s] in %s field: %s\n", name, second_str, grib_get_error_message(err));
         save_error(c, name);
         return err;
     }
@@ -754,7 +718,7 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
 
     if ((err = grib_get_size(handle1, name, &len1)) != GRIB_SUCCESS) {
         printInfo(handle1);
-        printf("Oops... cannot get size of [%s] in %s field: %s\n", name, first_str, grib_get_error_message(err));
+        printf("Error: cannot get size of [%s] in %s field: %s\n", name, first_str, grib_get_error_message(err));
         save_error(c, name);
         return err;
     }
@@ -768,7 +732,7 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
         }
 
         printInfo(handle1);
-        printf("Oops... cannot get size of [%s] in %s field: %s\n", name, second_str, grib_get_error_message(err));
+        printf("Error: cannot get size of [%s] in %s field: %s\n", name, second_str, grib_get_error_message(err));
         save_error(c, name);
         return err;
     }
@@ -784,21 +748,18 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
           return GRIB_COUNT_MISMATCH;
         } */
 
-    if (options->mode != MODE_BUFR) {
+    Assert(options->mode == MODE_BUFR);
         /* TODO: Ignore missing values for keys in BUFR. Not yet implemented */
-        isMissing1 = ((grib_is_missing(handle1, name, &err1) == 1) && (err1 == 0)) ? 1 : 0;
-        isMissing2 = ((grib_is_missing(handle2, name, &err2) == 1) && (err2 == 0)) ? 1 : 0;
-    }
+        //isMissing1 = ((grib_is_missing(handle1, name, &err1) == 1) && (err1 == 0)) ? 1 : 0;
+        //isMissing2 = ((grib_is_missing(handle2, name, &err2) == 1) && (err2 == 0)) ? 1 : 0;
 
     if ((isMissing1 == 1) && (isMissing2 == 1)) {
-        if (verbose)
-            printf(" is set to missing in both fields\n");
+        // if (verbose) printf(" is set to missing in both fields\n");
         return GRIB_SUCCESS;
     }
 
     if (isMissing1 == 1) {
-        if (verbose)
-            printf(" is set to missing in %s field\n", first_str);
+        // if (verbose) printf(" is set to missing in %s field\n", first_str);
         printInfo(handle1);
         printf("%s is set to missing in %s field is not missing in %s field\n", name, first_str, second_str);
         err1 = GRIB_VALUE_MISMATCH;
@@ -807,8 +768,7 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
     }
 
     if (isMissing2 == 1) {
-        if (verbose)
-            printf(" is set to missing in %s field\n", first_str);
+        // if (verbose) printf(" is set to missing in %s field\n", first_str);
         printInfo(handle1);
         printf("%s is set to missing in %s field is not missing in %s field\n", name, second_str, first_str);
         err1 = GRIB_VALUE_MISMATCH;
@@ -833,13 +793,13 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
                 sval2         = (char*)grib_context_malloc(handle2->context, slen2 * sizeof(char));
                 if ((err1 = grib_get_string(handle1, name, sval1, &slen1)) != GRIB_SUCCESS) {
                     printInfo(handle1);
-                    printf("Oops... cannot get string value of [%s] in %s field: %s\n",
+                    printf("Error: cannot get string value of [%s] in %s field: %s\n",
                            name, first_str, grib_get_error_message(err1));
                     save_error(c, name);
                 }
                 if ((err2 = grib_get_string(handle2, name, sval2, &slen2)) != GRIB_SUCCESS) {
                     printInfo(handle1);
-                    printf("Oops... cannot get string value of [%s] in %s field: %s\n",
+                    printf("Error: cannot get string value of [%s] in %s field: %s\n",
                            name, second_str, grib_get_error_message(err2));
                     save_error(c, name);
                 }
@@ -862,13 +822,13 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
 
                 if ((err1 = grib_get_string_array(handle1, name, svals1, &len1)) != GRIB_SUCCESS) {
                     printInfo(handle1);
-                    printf("Oops... cannot get string value of [%s] in %s field: %s\n",
+                    printf("Error: cannot get string value of [%s] in %s field: %s\n",
                            name, first_str, grib_get_error_message(err1));
                     save_error(c, name);
                 }
                 if ((err2 = grib_get_string_array(handle2, name, svals2, &len2)) != GRIB_SUCCESS) {
                     printInfo(handle1);
-                    printf("Oops... cannot get string value of [%s] in %s field: %s\n",
+                    printf("Error: cannot get string value of [%s] in %s field: %s\n",
                            name, second_str, grib_get_error_message(err2));
                     save_error(c, name);
                 }
@@ -917,14 +877,14 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
 
             if ((err1 = grib_get_long_array(handle1, name, lval1, &len1)) != GRIB_SUCCESS) {
                 printInfo(handle1);
-                printf("Oops... cannot get long value of [%s] in %s field: %s\n",
+                printf("Error: cannot get long value of [%s] in %s field: %s\n",
                        name, first_str, grib_get_error_message(err1));
                 save_error(c, name);
             }
 
             if ((err2 = grib_get_long_array(handle2, name, lval2, &len2)) != GRIB_SUCCESS) {
                 printInfo(handle1);
-                printf("Oops... cannot get long value of [%s] in %s field: %s\n",
+                printf("Error: cannot get long value of [%s] in %s field: %s\n",
                        name, second_str, grib_get_error_message(err2));
                 save_error(c, name);
             }
@@ -1002,14 +962,14 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
 
             if ((err1 = grib_get_double_array(handle1, name, dval1, &len1)) != GRIB_SUCCESS) {
                 printInfo(handle1);
-                printf("Oops... cannot get double value of [%s] in %s field: %s\n",
+                printf("Error: cannot get double value of [%s] in %s field: %s\n",
                        name, first_str, grib_get_error_message(err1));
                 save_error(c, name);
             }
 
             if ((err2 = grib_get_double_array(handle2, name, dval2, &len2)) != GRIB_SUCCESS) {
                 printInfo(handle1);
-                printf("Oops... cannot get double value of [%s] in %s field: %s\n",
+                printf("Error: cannot get double value of [%s] in %s field: %s\n",
                        name, second_str, grib_get_error_message(err2));
                 save_error(c, name);
             }
@@ -1020,28 +980,13 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
                 save_error(c, name);
             }
             if (err1 == GRIB_SUCCESS && err2 == GRIB_SUCCESS && len1 == len2) {
-                int imaxdiff;
+                int imaxdiff = 0;
                 double diff;
                 double *pv1, *pv2;
                 maxdiff   = 0;
-                imaxdiff  = 0;
                 countdiff = 0;
                 pv1       = dval1;
                 pv2       = dval2;
-                /* if (isangle) {
-                    dnew1 = *dval1;
-                    dnew2 = *dval2;
-                    pv1   = &dnew1;
-                    pv2   = &dnew2;
-                    if (*dval1 < 0)
-                        dnew1 += 360.0;
-                    if (*dval2 < 0)
-                        dnew2 += 360.0;
-                    if (*dval1 > 360)
-                        dnew1 -= 360.0;
-                    if (*dval2 > 360)
-                        dnew2 -= 360.0;
-                } */
                 value_tolerance *= tolerance_factor;
                 if (verbose)
                     printf("  (%d values) tolerance=%g\n", (int)len1, value_tolerance);
@@ -1080,8 +1025,6 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
                             grib_context_free(c, svalA);
                             grib_context_free(c, svalB);
                         }
-                        if (packingError2 != 0 || packingError1 != 0)
-                            printf(" packingError: [%g] [%g]", packingError1, packingError2);
 
                         printf("\n");
                     }
@@ -1114,69 +1057,14 @@ static int compare_values(grib_runtime_options* options, grib_handle* handle1, g
             break;
 
         case GRIB_TYPE_BYTES:
-            if (verbose)
-                printf(" as bytes\n");
-            if (options->mode == MODE_BUFR)
-                return 0;
-            if (len1 < 2)
-                len1 = 512;
-            if (len2 < 2)
-                len2 = 512;
-            uval1 = (unsigned char*)grib_context_malloc(handle1->context, len1 * sizeof(unsigned char));
-            uval2 = (unsigned char*)grib_context_malloc(handle2->context, len2 * sizeof(unsigned char));
-
-            if ((err1 = grib_get_bytes(handle1, name, uval1, &len1)) != GRIB_SUCCESS) {
-                printInfo(handle1);
-                save_error(c, name);
-                printf("Oops... cannot get bytes value of [%s] in %s field: %s\n",
-                       name, first_str, grib_get_error_message(err1));
-            }
-
-            if ((err2 = grib_get_bytes(handle2, name, uval2, &len2)) != GRIB_SUCCESS) {
-                printInfo(handle1);
-                save_error(c, name);
-                printf("Oops... cannot get bytes value of [%s] in %s field: %s\n",
-                       name, second_str, grib_get_error_message(err2));
-            }
-
-            if (err1 == GRIB_SUCCESS && err2 == GRIB_SUCCESS) {
-                if (memcmp(uval1, uval2, len1) != 0) {
-                    for (i = 0; i < len1; i++) {
-                        if (uval1[i] != uval2[i]) {
-                            printInfo(handle1);
-                            save_error(c, name);
-                            if (len1 == 1)
-                                printf("[%s] byte values are different: [%02x] and [%02x]\n",
-                                       name, uval1[i], uval2[i]);
-                            else
-                                printf("[%s] byte value %d of %ld are different: [%02x] and [%02x]\n",
-                                       name, i, (long)len1, uval1[i], uval2[i]);
-
-                            err1 = GRIB_VALUE_MISMATCH;
-                            break;
-                        }
-                    }
-                    err1 = GRIB_VALUE_MISMATCH;
-                }
-            }
-
-            grib_context_free(handle1->context, uval1);
-            grib_context_free(handle2->context, uval2);
-
-            if (err1)
-                return err1;
-            if (err2)
-                return err2;
+            return 0; // Not in BUFR
             break;
 
         case GRIB_TYPE_LABEL:
-            if (verbose)
-                printf(" as label\n");
             break;
 
         default:
-            if (verbose)
-                printf("\n");
+            if (verbose) printf("\n");
             printInfo(handle1);
             save_error(c, name);
             printf("Cannot compare [%s], unsupported type %d\n", name, type1);
@@ -1190,22 +1078,22 @@ static int compare_attributes(grib_handle* handle1, grib_handle* handle2, grib_r
                               grib_accessor* a, const char* prefix, int* err)
 {
     int i = 0, ret = 0;
-    while (i < MAX_ACCESSOR_ATTRIBUTES && a->attributes[i]) {
+    while (i < MAX_ACCESSOR_ATTRIBUTES && a->attributes_[i]) {
         /*long native_type = 0;*/
         grib_accessor* aa = NULL;
-        if ((a->attributes[i]->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0) {
+        if ((a->attributes_[i]->flags_ & GRIB_ACCESSOR_FLAG_DUMP) == 0) {
             ++i; /* next attribute if accessor is not for dumping */
             continue;
         }
-        if ((a->attributes[i]->flags & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0) {
+        if ((a->attributes_[i]->flags_ & GRIB_ACCESSOR_FLAG_READ_ONLY) != 0) {
             ++i; /* next attribute if accessor is read-only */
             continue;
         }
 
-        aa = a->attributes[i];
+        aa = a->attributes_[i];
         /*native_type = grib_accessor_get_native_type(aa);   TODO: read only check? */
 
-        isLeafKey = aa->attributes[0] == NULL ? 1 : 0; /* update global variable */
+        isLeafKey = aa->attributes_[0] == NULL ? 1 : 0; /* update global variable */
 
         if (compare_attribute(handle1, handle2, options, aa, prefix, err)) {
             (*err)++;
@@ -1223,9 +1111,9 @@ static int compare_attribute(grib_handle* handle1, grib_handle* handle2, grib_ru
 {
     int ret         = 0;
     grib_context* c = handle1->context;
-    const size_t fullnameMaxLen = strlen(a->name) + strlen(prefix) + 5;
+    const size_t fullnameMaxLen = strlen(a->name_) + strlen(prefix) + 5;
     char* fullname  = (char*)grib_context_malloc_clear(c, sizeof(char) * fullnameMaxLen);
-    snprintf(fullname, fullnameMaxLen, "%s->%s", prefix, a->name);
+    snprintf(fullname, fullnameMaxLen, "%s->%s", prefix, a->name_);
     if (compare_values(options, handle1, handle2, fullname, GRIB_TYPE_UNDEFINED)) {
         (*err)++;
         ret = 1;
@@ -1282,19 +1170,19 @@ static int compare_all_dump_keys(grib_handle* handle1, grib_handle* handle2, gri
         name      = grib_keys_iterator_get_name(iter);
         if (blocklisted(name))
             continue;
-        if (xa == NULL || (xa->flags & GRIB_ACCESSOR_FLAG_DUMP) == 0)
+        if (xa == NULL || (xa->flags_ & GRIB_ACCESSOR_FLAG_DUMP) == 0)
             continue;
 
         /* Get full name of key, e.g. '#2#windSpeed' or 'blockNumber' */
-        rank = compute_bufr_key_rank(handle1, keys_list, xa->name);
+        rank = compute_bufr_key_rank(handle1, keys_list, xa->name_);
         if (rank != 0) {
-            const size_t prefixMaxLen = strlen(xa->name) + 10;
+            const size_t prefixMaxLen = strlen(xa->name_) + 10;
             prefix = (char*)grib_context_malloc_clear(context, sizeof(char) * prefixMaxLen);
             dofree = 1;
-            snprintf(prefix, prefixMaxLen, "#%d#%s", rank, xa->name);
+            snprintf(prefix, prefixMaxLen, "#%d#%s", rank, xa->name_);
         }
         else {
-            prefix = (char*)xa->name;
+            prefix = (char*)xa->name_;
         }
 
         if (blocklisted(prefix)) {
@@ -1335,12 +1223,11 @@ static int compare_handles(grib_handle* handle1, grib_handle* handle2, grib_runt
             if (options->compare[i].type == CODES_NAMESPACE) {
                 iter = grib_keys_iterator_new(handle1, 0, options->compare[i].name);
                 if (!iter) {
-                    grib_context_log(handle1->context, GRIB_LOG_ERROR, "unable to get iterator");
+                    grib_context_log(handle1->context, GRIB_LOG_ERROR, "unable to create the BUFR keys iterator");
                     exit(1);
                 }
                 while (grib_keys_iterator_next(iter)) {
                     name = grib_keys_iterator_get_name(iter);
-                    /*printf("----- comparing %s\n",name);*/
 
                     if (blocklisted(name))
                         continue;
@@ -1359,11 +1246,9 @@ static int compare_handles(grib_handle* handle1, grib_handle* handle2, grib_runt
     else {
         const void *msg1 = NULL, *msg2 = NULL;
         size_t size1 = 0, size2 = 0;
-        int memcmp_ret = 0;
-        /* int ii=0; */
         GRIB_CHECK_NOLINE(grib_get_message(handle1, &msg1, &size1), 0);
         GRIB_CHECK_NOLINE(grib_get_message(handle2, &msg2, &size2), 0);
-        if (size1 == size2 && !(memcmp_ret = memcmp(msg1, msg2, size1))) {
+        if (size1 == size2 && 0 == memcmp(msg1, msg2, size1)) {
             return 0;
         }
 #if defined(BUFR_COMPARE_BYTES)
@@ -1396,12 +1281,11 @@ static int compare_handles(grib_handle* handle1, grib_handle* handle2, grib_runt
                     iter = grib_keys_iterator_new(handle1, 0, options->compare[i].name);
                     if (!iter) {
                         grib_context_log(handle1->context, GRIB_LOG_ERROR,
-                                         "ERROR: unable to get keys iterator for %s", options->compare[i].name);
+                                         "unable to create the BUFR keys iterator for %s", options->compare[i].name);
                         exit(1);
                     }
                     while (grib_keys_iterator_next(iter)) {
                         name = grib_keys_iterator_get_name(iter);
-                        /*printf("----- comparing %s\n",name);*/
 
                         if (blocklisted(name))
                             continue;
